/* classe Bezier: calcola e disegna una curva di Bezier nel piano xy */
import java.awt.*;
import java.util.Vector;

class Bezier {
	Bezier(Vector punti, int iterazioni, Graphics g) {
		int i;
		float x,y;
		Grafica grafica = new Grafica(g);		
		
		// disegna la curva
		grafica.impostaColore(Color.black);
		grafica.vaiA(bezier(0,punti)); // punto iniziale, per u==0
		for (i=1; i<=iterazioni; i++) {
			grafica.disegnaLineaA(bezier(((float)i)/iterazioni,punti));
		}
	}
	
	private int bin(int n,int i) { // funzione binomiale
		float j;
		float b=1;
		
		for (j=1; j<=n-i; j++)
			b*=((j+i)/j);
		return (int) b;
	}
	
	private Coordinate2D bezier(float u,Vector punti) { // calcola il valore della curva di Bezier nel punto
		int i;
		double b,x=0,y=0;
		int grado=punti.size()-1;
		
		for (i=0; i<=grado; i++) {
			b = bin(grado,i) * Math.pow(1-u,(float)grado-i) * Math.pow(u,(float) i); // calcola il polinomio di Bernstein
			x += (((Coordinate2D)punti.elementAt(i)).x)*b;
			y += (((Coordinate2D)punti.elementAt(i)).y)*b;
		}
		return new Coordinate2D((int)x,(int)y);
	}
}