import java.awt.*;
import java.util.Vector;

class Disegno extends Panel {
   public static final int AGG_PUNTO = 0;
	public static final int MUOVI_PUNTO = 1;
	public static final int CANC_PUNTO = 2;
	private int modPunti = AGG_PUNTO;
	private Vector punti = new Vector(16,4);
   private int puntoInMov;
  	public static final int BEZIER = 0;
	public static final int BSPLINE = 1;
	private int tipoCurva = BEZIER;
   private int continuita=3;
   private boolean disegnaCurva=false;
   private boolean ridisegnaAutomaticamente=true;
   private int iterazioni;	
	
   Disegno() {
		setBackground(Color.white);
	}
	
   void impostaContinuita(int cont) {
      if (cont>=0) continuita=cont; // controllo negativo
      repaint();
	}
	
   String leggiContinuita() {
      return String.valueOf(continuita);
   }
	
   void disegna() {
      disegnaCurva=true;
      repaint();
	}
	
   void impostaRidisAut(boolean dis) {
      ridisegnaAutomaticamente=dis;
      repaint();
	}
	
   boolean leggiRidisAut() {
      return ridisegnaAutomaticamente;      
	}
	
	void impostaModPunti(int mod) {
		modPunti=mod;
	}
	
	void impostaTipoCurva(int tipo) {
		tipoCurva=tipo;
		repaint();
   }
	
	void impostaIterazioni(int iter) {
		if (iter>0) iterazioni=iter;
      repaint();	   
	}
	
	void eliminaPunti() {
		punti.removeAllElements();
      repaint();
	}
	
	private int individuaPuntoA(int x, int y) {
		for(int i=0; i<punti.size(); i++)
			if (((PuntoDiControllo2D)punti.elementAt(i)).vicinoA(x,y))
			return i;
		/*else*/ return -1;
	}
	
	public boolean handleEvent(Event ev) {
		switch (ev.id) {
		case Event.MOUSE_DOWN:
			switch (modPunti) {
			case AGG_PUNTO:
				punti.addElement(new PuntoDiControllo2D(ev.x,ev.y));
				repaint();
				break;
			case MUOVI_PUNTO:
				puntoInMov=individuaPuntoA(ev.x,ev.y);
				break;
			case CANC_PUNTO:
				int idPunto=individuaPuntoA(ev.x,ev.y);
				if(idPunto>=0) {
					punti.removeElementAt(idPunto);
					repaint();
				}
				break;
			default:
				throw new IllegalArgumentException();
			}
			return true;
			case Event.MOUSE_UP:
				if (puntoInMov>=0) {
					puntoInMov=-1;
					repaint();
				}
				return true;
			case Event.MOUSE_DRAG:
				if (puntoInMov>=0) {
					((PuntoDiControllo2D) punti.elementAt(puntoInMov)).x=ev.x;
               ((PuntoDiControllo2D) punti.elementAt(puntoInMov)).y=ev.y;
               repaint();
				}
				return true;
			default:
				return false;
		}
	}
	
	public void paint(Graphics g) {
      int i;
      g.setColor(getForeground());
		g.setPaintMode();
		g.drawRect(0,0, size().width-1, size().height-1);
		
      // disegna il poligono di controllo
      Grafica grafica = new Grafica(g);		
		grafica.impostaColore(Color.orange);
      for (i=0; i<punti.size()-1; i++)
         grafica.disegnaLinea(((Coordinate2D)punti.elementAt(i)),((Coordinate2D)punti.elementAt(i+1)));
		
      for (i=0; i<punti.size(); i++) {
			PuntoDiControllo2D punto = (PuntoDiControllo2D)punti.elementAt(i);
			punto.disegna(g,String.valueOf(i));			
		}
		
      if (ridisegnaAutomaticamente || disegnaCurva) 
         switch (tipoCurva) {
			case BEZIER:
				new Bezier(punti,iterazioni,g);
				break;
			case BSPLINE:
				new Bspline(punti,continuita,iterazioni,g);
				break;
			default:
				/*niente*/
      }
      disegnaCurva=false;
	}
}